const config = require('config');
const {createLogger, format, transports} = require('winston');
const {combine, timestamp, printf, colorize} = format;

const myFormat = printf(info => {
    return config.development ?
        `${info.timestamp}: [${info.level}] ${info.message}` : `[${info.level}] ${info.message}`;
});
const logFormat = config.development ? combine(timestamp(), colorize(), myFormat) : combine(myFormat);
const myLogger = createLogger({
    format: logFormat,
    transports: [new transports.Console({level: config.logLevel})]
});

module.exports = {
    logger: (req, res, next) => {
        req._startTime = Date.now();
        let end = res.end;
        res.end = function (chunk, encoding) {
            res.responseTime = Date.now() - req._startTime;

            res.end = end;
            res.end(chunk, encoding);

            let json = {
                method: req.method,
                url: req.url,
                headers: req.headers,
                query: req.query,
                body: req.body
            };

            try {
                myLogger.info(`${res.statusCode} - ${res.responseTime}ms - ${JSON.stringify(json)}`);
            } catch (e) {
                myLogger.error(e);
            }
        };

        next();
    },
    log: myLogger
};